/*
 * Created on Jan 7, 2005
 */
package gov.grants.apply.util;

import gov.grants.apply.Log4JInit;
import gov.grants.apply.xpath.GrantApplicationXpath;
import gov.grants.apply.xpath.XPathExecutorTest;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;

import javax.xml.parsers.DocumentBuilderFactory;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;

/**
 * @author David Wong
 * 
 * A unit test class for the GrantApplicationHash object.
 */
public class GrantApplicationHashTest extends TestCase {
	private static final String HASH_VALUE = "b3uroqqoGuv2kG4ux9VWJ+n0stM=";
	private static final String BYTE_HASH_VALUE = "/FdbRmVNe971RV88VMalWrOwZfE=";
												   
	private final static Logger LOG = Logger
			.getLogger(GrantApplicationHashTest.class.getName());

	private static Log4JInit init = new Log4JInit();

	private String _xml = null;

	private Document _document = null;
	
	private GrantApplicationXpath _xpath = null;

	public static void main(String[] args) {
		junit.textui.TestRunner.run(GrantApplicationHashTest.class);
	}

	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		Reader testFile = null;
		StringBuffer stringform = null;
		try {
			testFile = new InputStreamReader(new BufferedInputStream(
					XPathExecutorTest.class.getClassLoader()
							.getResourceAsStream("resources/testFile.xml")));
			stringform = new StringBuffer();
			int maxSize = 1024;
			char[] buffer = new char[maxSize];
			int charsRead = 0;
			while ((charsRead = testFile.read(buffer, 0, maxSize)) > -1) {
				stringform.append(buffer, 0, charsRead);
			}
		} finally {
			try {
				if (testFile != null) {
					testFile.close();
				}
			} catch (IOException e) {
			}
		}
		_xml = stringform.toString();
		InputStream stream = null;
		LOG.debug(_xml);
		try {
			//stream = new ByteArrayInputStream(_xml.getBytes());
			stream = new BufferedInputStream(XPathExecutorTest.class
					.getClassLoader().getResourceAsStream(
							"resources/testFile.xml"));
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			factory.setNamespaceAware(true);
			_document = factory.newDocumentBuilder().parse(stream);
		} finally {
			try {
				stream.close();
			} catch (Exception e) {
				//ignore
			}
		}
		_xpath = new GrantApplicationXpath(_xml);
	}

	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Constructor for GrantApplicationHashTest.
	 * 
	 * @param arg0
	 */
	public GrantApplicationHashTest(String arg0) {
		super(arg0);
	}

	public final void testHashXml() {
		try {
			LOG.debug(GrantApplicationHash.computeGrantFormsHash(_xml));
			Assert.assertEquals("Hash values did not match.", HASH_VALUE,
					GrantApplicationHash.computeGrantFormsHash(_xml));
		} catch (Exception e) {
			Assert.fail(e.getMessage());
		}
	}

	public final void testHashDocument() {
		try {
			Assert.assertEquals("Hash values did not match.", HASH_VALUE,
					GrantApplicationHash.computeGrantFormsHash(_document));
		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		}
	}

	public final void testHashXpath() {
		try {
			Assert.assertEquals("Hash values did not match.", HASH_VALUE,
					GrantApplicationHash.computeGrantFormsHash(_xpath));
		} catch (Exception e) {
			Assert.fail(e.getMessage());
		}
	}
	
	public final void testHashAttachment(){
		try {
			
			Assert.assertEquals("Hash values did not match.", BYTE_HASH_VALUE,
					GrantApplicationHash.computeAttachmentHash( _xml.getBytes() ));
			
		} catch (Exception e) {
			Assert.fail(e.getMessage());
		}
	}
	
	
}