/*
 * Created on Jun 1, 2005
 *
 * Unpublished Work, Copyright, 2004 Northrop Grumman
 */
package gov.grants.apply.dao.xmlDAO;

import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GrantsGovApplicationStatusType;
import gov.grants.apply.dao.Application;
import gov.grants.apply.dao.DAOException;
import gov.grants.apply.dao.GetApplicationListFilter;
import gov.grants.apply.dao.IApplicationDAO;
import gov.grants.apply.dao.SF424Footer;
import gov.grants.apply.dao.SF424Header;
import gov.grants.apply.system.footer_v1.GrantSubmissionFooterType;
import gov.grants.apply.system.header_v1.GrantSubmissionHeaderType;
import gov.grants.apply.system.metagrantapplication.GrantApplication;
import gov.grants.apply.util.FileUtil;
import gov.grants.apply.util.PerformanceTimer;
import gov.grants.apply.util.WebAppContext;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;

import javax.activation.FileDataSource;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.apache.log4j.Logger;

/**
 * @author Grants.gov WebServices Team
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public final class XmlApplicationDAO implements IApplicationDAO
{
   private static final String GENERATED_JAXB_PACKAGE = "gov.grants.apply.system.metagrantapplication"; //"gov.grants.apply.jaxb";
   private final static Logger LOG = Logger.getLogger(XmlApplicationDAO.class);
   
   /**
    * Searches for applications according to the filter given.
    * @param filter Contains filter criteria for choosing which application to return.
    * @return An array of applications that match the filter criteria.
    */
   public Application[] findApplications(GetApplicationListFilter filter)
   	throws DAOException
   {
      PerformanceTimer.start();
      File samplesDirPath = new File(WebAppContext.getInstance().getRealPath("samples"));
      File[] samples = null;
      ArrayList apps = null;
      try
      {
         samples = FileUtil.findFilesWithExt(samplesDirPath, ".xml");
      }
      catch (FileNotFoundException e)
      {
         LOG.warn("Did not find any sample application XML files in web applicaiton.", e);
         return null;
      }
      try
      {
         JAXBContext jaxbContext = JAXBContext.newInstance(GENERATED_JAXB_PACKAGE);
         Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
         unmarshaller.setValidating(false);
         Object data = null;
         GrantApplication application = null;
         apps = new ArrayList(samples.length);
         for (int i = 0; i < samples.length; i++)
         {
            data = unmarshaller.unmarshal(samples[i]);
            if (LOG.isDebugEnabled())
            {
               LOG.debug("Object type unmarshalled by JAXB: " + data.getClass());
            }

            if (data instanceof GrantApplication)
            {
               application = (GrantApplication) data;
            }
            else
            {
               continue;
            }
            if (_matchesFilter(application, filter))
            {
               GrantSubmissionHeaderType jheader = application.getGrantSubmissionHeader();
               SF424Header header = new SF424Header();
               header.setActivityTitle(jheader.getActivityTitle());
               header.setAgencyName(jheader.getAgencyName());
               header.setCfdaNumber(jheader.getCFDANumber());
               header.setClosingDate(jheader.getClosingDate());
               header.setCompetitionID(jheader.getCompetitionID());
               header.setHashAlgorithm(jheader.getHashValue().getHashAlgorithm());
               header.setHashValue(jheader.getHashValue().getValue());
               header.setOpeningDate(jheader.getOpeningDate().getTime());
               header.setOpportunityID(jheader.getOpportunityID());
               header.setOpportunityTitle(jheader.getOpportunityTitle());
               header.setSchemaVersion(jheader.getSchemaVersion());
               header.setSubmissionTitle(jheader.getSubmissionTitle());
               SF424Footer footer = new SF424Footer();
               GrantSubmissionFooterType jfooter = application.getGrantSubmissionFooter();
               footer.setGrantsGovTrackingNumber(jfooter.getGrantsGovTrackingNumber());
               footer.setHashAlgorithm(jfooter.getHashValue().getHashAlgorithm());
               footer.setHashValue(jfooter.getHashValue().getValue());
               footer.setReceivedDateTime(jfooter.getReceivedDateTime());
               footer.setSchemaVersion(jfooter.getSchemaVersion());
               footer.setSubmitterName(jfooter.getSubmitterName());
               Application app = new Application();
               app.setHeader(header);
               app.setFooter(footer);
               app.setApplicationStatus(GrantsGovApplicationStatusType.value4); // Validated
               app.setXmlSource(new FileDataSource(samples[i]));
               apps.add(app);
            }
         }
      }
      catch (JAXBException e)
      {
         throw new DAOException(e);
      }
      if (apps != null)
      {
         PerformanceTimer.stop();
         apps.trimToSize();
         return (Application[]) apps.toArray(new Application[0]);
      }
      PerformanceTimer.stop();
      return null;
   }

   private boolean _matchesFilter(GrantApplication application, GetApplicationListFilter filter)
   {
      String trackingNum = filter.getGrantsGovTrackingNum();
      if (trackingNum != null)
      {
         return trackingNum.equals(application.getGrantSubmissionFooter().getGrantsGovTrackingNumber());
      }
      return true; // dummy code for now, just let everything pass.
   }
}
